﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Shizuku_Lib;
using System.IO.Ports;
using static Shizuku_Lib.ShizukuProtocolMeterData;
using static Shizuku_Lib.ShizukuProtocolFastCharge;

namespace Shizuku_Lib_Example
{
    /*
      This is an example to communicate with Shiuzku USB Multimeter with Shizuku Lib.
    */

    public partial class Form1 : Form
    {
        ShizukuProtocol protocolStack;
        SerialPort targetPort;
        ShizukuProtocolMeterData pMeterData;
        ShizukuProtocolFastCharge pFastCharge;
        USBMeterData_t USBMeterData_RealTime = new USBMeterData_t();

        private void MeterDataReportCallback(byte[] packet)
        {
            /*When a meter report packet is received, this method is called, it converts the packet into USBMeterData_RealTime*/
            byte[] payload = packet.Skip(9).Take(32).ToArray();
            ByteStructConverter byteStructConverter = new ByteStructConverter();
            USBMeterData_RealTime = (USBMeterData_t)byteStructConverter.BytesToStruct(payload, USBMeterData_RealTime.GetType());
        }

        ShizukuProtocol.CallerDescriptor.Callback_t mdr_cb;

        public Form1()
        {
            InitializeComponent();
        }

        private async void ConnectButton_Click(object sender, EventArgs e)
        {
            /* The following gives you a basic workflow of using the library.
             * After this, you should be able to fit the library into your own project.
            */

            /*First, you declare a SerialPort with the port No of the multimeter.
              Replace the port No to make the demo work.*/
            targetPort = new SerialPort("COM6");

            /*You pass the serial port object to construct a ShizukuProtocol object, which 
             handles low layer communications with the multimeter*/
            protocolStack = new ShizukuProtocol(targetPort);
            /*It's recommended to send a reset command before any operation*/
            await protocolStack.Reset();

            /*Then you can read the meter data*/
            pMeterData = new ShizukuProtocolMeterData(protocolStack);
            /*Specify a callback when a meter data report packet is received*/
            mdr_cb = new ShizukuProtocol.CallerDescriptor.Callback_t(MeterDataReportCallback);
            /*We request the meter to report meter data every 1000 miliseconds*/
            await pMeterData.MeterDataReport_Setup(1000, mdr_cb, false);

            /*This is how you trigger fastcharge.*/
            pFastCharge = new ShizukuProtocolFastCharge(protocolStack);
            await pFastCharge.FastChargeCommand(new UInt32[1] { (uint)FastChargeCommandType.CREATE_INTERFACE }, true);
            /*Send a command ENTER_MODE with a protocol paramater, we use QC2.0 as an example*/
            await pFastCharge.FastChargeCommand(
                new UInt32[2] { (uint)FastChargeCommandType.ENTER_MODE,
                (uint)FastChargeMode.QC_2P0}, false);
            /*Then you request to trigger a certain voltage, in this case we want 9V@3A*/
            await pFastCharge.FastChargeCommand(
                          new UInt32[3] { (uint)FastChargeCommandType.CHANGE_VALUE
                     ,9000
                     ,3000},false);
            /*You can plug-in a QC2.0 supported charger to see the result*/

            /*There is little documentation, contact +8618628228557 for further support*/
            /*If there is a demand, we will make a full documentation of more APIs.*/
        }
    }
}

